@@ -1,10 +1,10 @@
 package Net::DNS::RR;
 
 #
-# $Id: RR.pm 1203 2014-05-20 12:25:01Z willem $
+# $Id: RR.pm 1260 2014-09-09 09:12:28Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1203 $)[1];
+$VERSION = (qw$LastChangedRevision: 1260 $)[1];
 
 
 =head1 NAME
@@ -32,20 +32,22 @@ See also the manual pages for each specific RR type.
 =cut
 
 
-use constant COMPATIBLE => eval {	## enable architecture transition code
-	require Net::DNS::RR::DS;	## Net::DNS::SEC 0.17 compatible
-	$Net::DNS::RR::DS::VERSION < 1179;
-} || 0;
-
-
 use strict;
 use integer;
 use Carp;
 
 use Net::DNS::Parameters;
+use Net::DNS::Domain;
 use Net::DNS::DomainName;
 
 
+use constant COMPATIBLE => eval {	## enable architecture transition code
+	return 0 if $] < 5.006;
+	require Net::DNS::RR::DS;	## Net::DNS::SEC 0.17 compatible
+	( $Net::DNS::RR::DS::VERSION || 0 ) < 1133;
+} || 0;
+
+
 =head1 METHODS
 
 B<WARNING!!!>  Do not assume the RR objects you receive from a query
@@ -58,12 +60,14 @@ you will get an error message and execution will be terminated.
 sub new {
 	return &_new_from_rdata if COMPATIBLE && ref $_[1];	# resolve new() usage conflict
 
-	return eval { scalar @_ > 2 ? &_new_hash : &_new_string; } || do {
-		my $error = $@	  || 'eval{} aborted without setting $@, contrary to Perl specification' . "\n";
+	return eval {
+		local $SIG{__WARN__} = sub { die @_ };
+		scalar @_ > 2 ? &_new_hash : &_new_string;
+	} || do {
 		my $class = shift || __PACKAGE__;
-		my @parse = split /\s+/, shift || '';
-		croak join ' ', "${error}in new $class(", substr( "@parse @_", 0, 50 ), '... )';
-			}
+		my @param = map { !defined($_) ? 'undef' : split; } @_;
+		croak join ' ', "$@in new $class(", substr( "@param", 0, 50 ), '... )';
+	};
 }
 
 
@@ -91,7 +95,7 @@ The trailing dot (.) is optional.
 
 =cut
 
-my $PARSE_REGEX = q/("[^"]*"|'[^']*')|;[^\n]*|[\s()]/;
+my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]/;
 
 sub _new_string {
 	my $base;
@@ -101,8 +105,7 @@ sub _new_string {
 
 	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
 	s/\\\\/\\092/g;						# disguise escaped escape
-	s/\\"/\\034/g;						# disguise escaped double quote
-	s/\\'/\\039/g;						# disguise escaped single quote
+	s/\\"/\\034/g;						# disguise escaped quote
 	s/\\\(/\\040/g;						# disguise escaped bracket
 	s/\\\)/\\041/g;						# disguise escaped bracket
 	s/\\;/\\059/g;						# disguise escaped semicolon
@@ -113,17 +116,14 @@ sub _new_string {
 	croak 'unable to parse RR string' unless defined $t1;
 
 	my ( $ttl, $class );
-	unless ( defined $t2 ) {
-		$token[0] = 'ANY' unless $typebyname{$t1};	# <name> <type>
-	} elsif ( defined $t3 && $classbyname{$t2} ) {
-		$ttl   = shift @token;				# <name> <ttl> <class> <type>
-		$class = shift @token;
-	} elsif ( $t1 =~ /^\d/ ) {
-		$ttl = shift @token;				# <name> <ttl> [<class>] <type>
-		$class = shift @token if $t2 =~ /^CLASS\d/;
+	unless ( defined $t2 ) {				# <name> <type>
+		@token = ('ANY') if $classbyname{$t1};		# <name> <class>
 	} elsif ( $classbyname{$t1} || $t1 =~ /^CLASS\d/ ) {
 		$class = shift @token;				# <name> <class> [<ttl>] <type>
 		$ttl = shift @token if $t2 =~ /^\d/;
+	} elsif ( $t1 =~ /^\d/ ) {
+		$ttl = shift @token;				# <name> <ttl> [<class>] <type>
+		$class = shift @token if $classbyname{$t2} || $t2 =~ /^CLASS\d/;
 	}
 
 	my $type      = shift(@token);
@@ -136,7 +136,7 @@ sub _new_string {
 
 	return $self unless $populated;				# empty RR
 
-	if ( $token[0] =~ /^[\\]?#$/ ) {
+	if ( $#token && $token[0] =~ /^[\\]?#$/ ) {
 		shift @token;					# RFC3597 hexadecimal format
 		my $count = shift(@token) || 0;
 		my $rdata = pack 'H*', join '', @token;
@@ -350,9 +350,50 @@ sub canonical {
 }
 
 
-=head2 name
+=head2 print
 
-    $name = $rr->name;
+    $rr->print;
+
+Prints the record to the standard output.  Calls the B<string>
+method to get the RR string representation.
+
+=cut
+
+sub print {
+	print shift->string, "\n";
+}
+
+
+=head2 string
+
+    print $rr->string, "\n";
+
+Returns a string representation of the RR using the zone file format
+described in RFC1035.  All domain names are fully qualified with
+trailing dot.  This differs from RR attribute methods, which omit
+the trailing dot.
+
+=cut
+
+sub string {
+	my $self = shift;
+
+	my $name = $self->name if COMPATIBLE;
+	my @core = ( $self->{owner}->string, $self->ttl, $self->class, $self->type );
+
+	my $rdata = $self->rdstring;
+
+	return join "\t", @core, '; no data' unless length $rdata;
+
+	chomp $rdata;
+	$rdata =~ s/\n+/\n\t/g;
+	return join "\t", @core, $rdata;
+}
+
+
+=head2 owner name
+
+    $owner = $rr->name;
 
 Returns the owner name of the record.
 
@@ -371,6 +412,8 @@ sub name {
 	$self->{owner}->name if defined wantarray;
 }
 
+sub owner { &name; }			## compatibility with RFC1034
+
 
 =head2 type
 
@@ -461,47 +504,6 @@ sub rdata {
 }
 
 
-=head2 print
-
-    $rr->print;
-
-Prints the record to the standard output.  Calls the B<string>
-method to get the RR string representation.
-
-=cut
-
-sub print {
-	print shift->string, "\n";
-}
-
-
-=head2 string
-
-    print $rr->string, "\n";
-
-Returns a string representation of the RR using the zone file format
-described in RFC1035.  All domain names are fully qualified with
-trailing dot.  This differs from RR attribute methods, which omit
-the trailing dot.
-
-=cut
-
-sub string {
-	my $self = shift;
-
-	my $name = $self->name if COMPATIBLE;
-	my @core = ( $self->{owner}->string, $self->ttl, $self->class, $self->type );
-
-	my $rdata = $self->rdstring;
-
-	return join "\t", @core, '; no data' unless length $rdata;
-
-	chomp $rdata;
-	$rdata =~ s/\n+/\n\t/g;
-	return join "\t", @core, $rdata;
-}
-
-
 =head2 rdstring
 
     $rdstring = $rr->rdstring;
@@ -516,10 +518,10 @@ sub rdstring {
 	my $rdata = eval {
 		return $self->rdatastr if COMPATIBLE;
 		return $self->format_rdata;
-	} || '';
+	};
 	carp $@ if $@;
 
-	return $rdata;
+	return defined $rdata ? $rdata : '';
 }
 
 
@@ -547,16 +549,15 @@ Returns a token list representation of the RR zone file string.
 =cut
 
 sub token {
-	my $self = shift;
+	local $_ = shift->string;
 
-	my @core = ( $self->{owner}->string, $self->ttl, $self->class, $self->type );
-	local $_ = $self->rdstring;
+	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
 	s/\\\\/\\092/g;						# disguise escaped escape
-	s/\\"/\\034/g;						# disguise escaped double quote
-	s/\\'/\\039/g;						# disguise escaped single quote
+	s/\\"/\\034/g;						# disguise escaped quote
+	s/\\\(/\\040/g;						# disguise escaped bracket
+	s/\\\)/\\041/g;						# disguise escaped bracket
 	s/\\;/\\059/g;						# disguise escaped semicolon
-	my @parse = grep defined && length, split /$PARSE_REGEX/o;
-	my @token = @core, grep !/^[()]$/, @parse;		# discard brackets
+	my @token = grep defined && length, split /$PARSE_REGEX/o;
 }
 
 
@@ -651,9 +652,9 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 
 sub format_rdata {			## format rdata portion of RR string
 	my $self = shift;
-	my $data = $self->{'rdata'} || $self->encode_rdata;	# unknown RR, per RFC3597
-	my $size = length $data;
-	join ' ', '\\#', $size, $size ? unpack( 'H*', $data ) : ();
+	my $data = $self->{'rdata'} || $self->encode_rdata;
+	my $size = length($data) || return '';
+	join ' ', '\\#', $size, unpack 'H*', $data;		# RFC3597 unknown RR format
 }
 
 
@@ -703,7 +704,6 @@ sub _subclass {
 		my $module = join '::', $class, $mnemon;
 		$module =~ s/[^A-Za-z0-9:]//g;			# expect the unexpected
 		$subclass = eval("require $module") ? $module : $class;
-		$subclass = $module if $mnemon eq 'OPT';	# default to OPT declared below
 		my $object = bless {'type' => $number}, $subclass;
 		if (COMPATIBLE) {
 			no strict;
@@ -711,13 +711,16 @@ sub _subclass {
 			$object->{OLD}++ unless exists $stash{'encode_rdata'};
 			$object->{'type'} = $mnemon;
 		}
-		$_MINIMAL{$subclass} = [%$object];		# cache minimal content
-		$object->defaults if $subclass eq $module;
-		$_DEFAULT{$subclass} = [%$object];		# cache default content
-		$_LOADED{$rrtype}    = $subclass;
+
+		# cache pre-built minimal and populated default object images
+		$_MINIMAL{$rrtype} = $_MINIMAL{$mnemon} ||= [%$object];
+		$_DEFAULT{$rrtype} = $_DEFAULT{$mnemon} ||= do { $object->defaults; [%$object] };
+
+		$subclass = $module if $mnemon eq 'OPT';	# default to OPT declared below
+		$_LOADED{$rrtype} = $_LOADED{$mnemon} = $subclass;
 	}
 
-	my $prebuilt = $default ? $_DEFAULT{$subclass} : $_MINIMAL{$subclass};
+	my $prebuilt = $default ? $_DEFAULT{$rrtype} : $_MINIMAL{$rrtype};
 	return bless {@$prebuilt}, $subclass;			# create object
 }
 
@@ -784,7 +787,9 @@ sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
 sub AUTOLOAD {				## Default method
 	my $self = shift;
-	confess "method '$AUTOLOAD' undefined" unless ref $self;
+	my $oref = ref($self);
+	confess 'undefined method ', $AUTOLOAD unless $oref;
+	confess 'unimplemented type ', $self->type if $oref eq __PACKAGE__;
 
 	my $method = $1 if $AUTOLOAD =~ m/^.*::(.*)$/;
 
