@@ -1,10 +1,10 @@
 package Net::DNS::Resolver::MSWin32;
 
 #
-# $Id: MSWin32.pm 1202 2014-05-19 08:34:16Z willem $
+# $Id: MSWin32.pm 1258 2014-09-04 12:23:18Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1202 $)[1];
+$VERSION = (qw$LastChangedRevision: 1258 $)[1];
 
 =head1 NAME
 
@@ -17,7 +17,6 @@ use strict;
 use base qw(Net::DNS::Resolver::Base);
 
 use Carp;
-use Data::Dumper;
 
 BEGIN {
 	use vars qw($Registry);
@@ -36,34 +35,30 @@ BEGIN {
 }
 
 
+sub _untaint { map defined && /^(.+)$/ ? $1 : (), @_; }
+
+
 sub init {
+	my $defaults = shift->defaults;
 
 	my $debug = 0;
-	my ($class) = @_;
-
-	my $defaults = $class->defaults;
-
 
 	my $FIXED_INFO = {};
 
-	my $ret = Win32::IPHelper::GetNetworkParams($FIXED_INFO);
-
-	if ( $ret == 0 ) {
-		print Dumper $FIXED_INFO if $debug;
-	} else {
+	if ( my $ret = Win32::IPHelper::GetNetworkParams($FIXED_INFO) ) {
 		Carp::croak "GetNetworkParams() error %u: %s\n", $ret, Win32::FormatMessage($ret);
+	} elsif ($debug) {
+		require Data::Dumper;
+		print Data::Dumper::Dumper $FIXED_INFO;
 	}
 
 
 	my @nameservers = map { $_->{IpAddress} } @{$FIXED_INFO->{DnsServersList}};
-	$defaults->nameservers(@nameservers) if scalar @nameservers;
+	$defaults->nameservers( _untaint @nameservers );
 
-	my $domain = $FIXED_INFO->{DomainName} || '';
-	my $searchlist = $domain;
-	$defaults->{domain} = $domain if $domain;
-
-
-	my $usedevolution = 0;
+	my $devolution = 0;
+	my @searchlist = _untaint lc $FIXED_INFO->{DomainName};
+	$defaults->domain(@searchlist);
 
 	if (WINREG) {
 
@@ -81,49 +76,36 @@ sub init {
 		}
 
 		if ( defined $reg_tcpip ) {
-			$searchlist .= ',' if $searchlist;	# $domain already in there
-			$searchlist .= ( $reg_tcpip->GetValue('SearchList') || "" );
+			my $searchlist = lc $reg_tcpip->GetValue('SearchList') || '';
+			push @searchlist, split m/[\s,]+/, $searchlist;
+
 			my ( $value, $type ) = $reg_tcpip->GetValue('UseDomainNameDevolution');
-			$usedevolution = defined $value && $type == REG_DWORD ? hex $value : 0;
+			$devolution = defined $value && $type == REG_DWORD ? hex $value : 0;
 		}
 	}
 
 
-	if ($searchlist) {
+	# fix devolution if configured, and simultaneously
+	# make sure no dups (but keep the order)
+	my @list;
+	my %seen;
+	foreach my $entry (@searchlist) {
+		push( @list, $entry ) unless $seen{$entry}++;
 
-		# fix devolution if configured, and simultaneously
-		# make sure no dups (but keep the order)
-		my @a;
-		my %h;
-		foreach my $entry ( split( m/[\s,]+/, lc $searchlist ) ) {
-			push( @a, $entry ) unless $h{$entry}++;
+		next unless $devolution;
 
-			if ($usedevolution) {
-
-				# as long there are more than two pieces, cut
-				while ( $entry =~ m#\..+\.# ) {
-					$entry =~ s#^[^\.]+\.(.+)$#$1#;
-					push( @a, $entry ) unless $h{$entry}++;
-				}
-			}
+		# as long there are more than two pieces, cut
+		while ( $entry =~ m#\..+\.# ) {
+			$entry =~ s#^[^\.]+\.(.+)$#$1#;
+			push( @list, $entry ) unless $seen{$entry}++;
 		}
-		$defaults->{searchlist} = [@a];
 	}
+	$defaults->searchlist( _untaint @list );
 
-
-	$class->read_env;
-
-
-	if ( !$defaults->{domain} && @{$defaults->{searchlist}} ) {
-		$defaults->{domain} = $defaults->{searchlist}[0];
-	} elsif ( !@{$defaults->{searchlist}} && $defaults->{domain} ) {
-		$defaults->{searchlist} = [$defaults->{domain}];
-	}
-
-	print Dumper $defaults if $debug;
-
+	$defaults->read_env;
 }
 
+
 1;
 __END__
 
