@@ -1,11 +1,11 @@
 package Net::DNS::Resolver::UNIX;
-use base Net::DNS::Resolver::Base;
 
 #
-# $Id: UNIX.pm 1096 2012-12-28 13:35:15Z willem $
+# $Id: UNIX.pm 1235 2014-07-29 07:58:19Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1096 $)[1]; # Unchanged since 1021
+$VERSION = (qw$LastChangedRevision: 1235 $)[1];
+
 
 =head1 NAME
 
@@ -15,6 +15,7 @@ Net::DNS::Resolver::UNIX - UNIX Resolver Class
 
 
 use strict;
+use base qw(Net::DNS::Resolver::Base);
 
 
 my $resolv_conf = "/etc/resolv.conf";
@@ -25,25 +26,24 @@ push( @config_path, $ENV{HOME} ) if exists $ENV{HOME};
 push( @config_path, '.' );
 
 
-sub init {
-	my ($class) = @_;
+sub _untaint { map defined && /^(.+)$/ ? $1 : (), @_; }
 
-	$class->read_config_file($resolv_conf) if -f $resolv_conf && -r _;
+
+sub init {
+	my $defaults = shift->defaults;
+
+	$defaults->read_config_file($resolv_conf) if -f $resolv_conf && -r _;
+
+	$defaults->domain( _untaint $defaults->domain );	# untaint config values
+	$defaults->searchlist( _untaint $defaults->searchlist );
+	$defaults->nameservers( _untaint $defaults->nameservers );
 
 	foreach my $dir (@config_path) {
 		my $file = "$dir/$dotfile";
-		$class->read_config_file($file) if -f $file && -r _ && -o _;
+		$defaults->read_config_file($file) if -f $file && -r _ && -o _;
 	}
 
-	$class->read_env;
-
-	my $defaults = $class->defaults;
-
-	if ( !$defaults->{domain} && @{$defaults->{searchlist}} ) {
-		$defaults->{domain} = $defaults->{searchlist}[0];
-	} elsif ( !@{$defaults->{searchlist}} && $defaults->{domain} ) {
-		$defaults->{searchlist} = [$defaults->{domain}];
-	}
+	$defaults->read_env;
 }
 
 
